#include <stdio.h>

/* Must be multiple of 32, to keep BMP generator correct */
#define WIDTH0 2048
/* Real part: -2 .. 1 */
#define WIDTH 3*WIDTH0
/* Imaginary part: -1 .. 1 */
#define HEIGHT 2*WIDTH0
#define ITERATIONS 50

int is_in_set(double re, double im, int iterations)
{
    double zre, zim, zre2, zim2;
    zre = zim = 0.0;
    int i;
    for(i=0; i<iterations; i++) {
        /* z_n1 = z_n**2 + c */
        zre2 = zre*zre-zim*zim+re;
        zim2 = 2*zre*zim+im;
        zre = zre2;
        zim = zim2;
        /* if abs(z) > 2 */
        if (zre*zre + zim*zim > 4)
            return 0;
    }
    return 1;
}

/* http://de.wikipedia.org/wiki/Windows_Bitmap */
/* black-and-white 1bpp uncompressed 
*/
#define HDRSIZE (14 + 40 + 8)
#define IMGSIZE (HDRSIZE + WIDTH*HEIGHT/8)
#define NEGWIDTH ((unsigned int)-WIDTH)
#define LE(X) X % 256, X / 256 % 256, \
        X / 256 / 256 % 256,          \
        X / 256 / 256 / 256 % 256
unsigned char bmp[IMGSIZE] = {
        'B', 'M',                     /* bfType */
        LE(IMGSIZE),                  /* bfSize */
        0, 0, 0, 0,                   /* bfReserved */
        LE(HDRSIZE),                  /* bfOffBits */
        40, 0, 0, 0,                  /* biSize */
        LE(WIDTH),                    /* biWidth */
        LE(HEIGHT),                   /* biHeight*/
        1, 0,                         /* biPlanes */
        1, 0,                         /* biBitCount */
        0, 0, 0, 0,                   /* biCompression */
        LE(WIDTH*HEIGHT/8),           /* biSizeImage */
        0, 0, 0, 0,                   /* biXPelsPerMeter */
        0, 0, 0, 0,                   /* biYPelsPerMeter */
        0, 0, 0, 0,                   /* biClrUsed */
        0, 0, 0, 0,                   /* biClrImportant */
        /* color table */
        255, 255, 255, 0,             /* white */
        0, 0, 0, 0,                   /* black */
};

int main()
{
    FILE *out;
    int x, y, byte, bit;

    /* MANDEL begin */
    byte = HDRSIZE;
    bit = 0;
    for(y = -WIDTH0; y < WIDTH0; y++)
        for(x = -2*WIDTH0; x < WIDTH0; x++) {
	  bmp[byte] <<= 1;
            if (is_in_set(1.0*x/WIDTH0, 1.0*y/WIDTH0, ITERATIONS))
                bmp[byte]++;
            bit++;
            if (bit == 8) {
                byte++;
                bit = 0;
            }
        }
    /* MANDEL end */

    out = fopen("mandel.bmp", "wb");
    if (!out) {
        perror("Could not open file");
        return 1;
    }
    if (fwrite(bmp, 1, IMGSIZE, out) != IMGSIZE) {
        perror("Could not write bitmap data");
        return 1;
    }
    if (fclose(out) != 0) {
        perror("Closing failed");
        return 1;
    }
    return 0;
}
